package com.io.yutian.gemsystem;

import com.io.yutian.gemsystem.data.GemCraftData;
import org.bukkit.configuration.ConfigurationSection;
import org.bukkit.configuration.file.FileConfiguration;

import java.util.HashMap;
import java.util.Map;

public class GemSystemSetting {

    private static Map<String, Map<Integer, GemCraftData>> gemCraftDatas = new HashMap<>();

    public static void reload() {
        gemCraftDatas.clear();
        GemSystem.inst().saveDefaultConfig();
        GemSystem.inst().reloadConfig();
        FileConfiguration config = GemSystem.inst().getConfig();
        ConfigurationSection section = config.getConfigurationSection("gems");
        for (String key : section.getKeys(false)) {
            ConfigurationSection craftSection = section.getConfigurationSection(key);
            Map<Integer, GemCraftData> craftDatas = new HashMap<>();
            for (String craftKey : craftSection.getKeys(false)) {
                int level = Integer.parseInt(craftKey);
                double chance = craftSection.getDouble(craftKey+".chance");
                String result = craftSection.getString(craftKey+".result");
                String failResult = craftSection.getString(craftKey+".failResult");
                craftDatas.put(level, new GemCraftData(chance, result, failResult));
            }
            gemCraftDatas.put(key, craftDatas);
        }
    }

    public static Map<Integer, GemCraftData> getGemCraftDatas(String gemType) {
        return gemCraftDatas.get(gemType);
    }

    public static boolean hasGemCraftData(String gemType) {
        return gemCraftDatas.containsKey(gemType);
    }

    public static boolean hasGemCraftData(String gemType, int level) {
        if (!hasGemCraftData(gemType)) {
            return false;
        }
        Map<Integer, GemCraftData> craftDatas = gemCraftDatas.get(gemType);
        return craftDatas.containsKey(level);
    }

    public static GemCraftData getGemCraftData(String gemType, int level) {
        if (!hasGemCraftData(gemType)) {
            return null;
        }
        Map<Integer, GemCraftData> craftDatas = gemCraftDatas.get(gemType);
        if (!craftDatas.containsKey(level)) {
            return null;
        }
        return craftDatas.get(level);
    }

}
