package com.io.yutian.gemsystem.gui;

import com.io.yutian.gemsystem.GemSystemSetting;
import com.io.yutian.gemsystem.data.GemCraftData;
import com.io.yutian.gemsystem.util.PlayerInventoryUtil;
import com.io.yutian.gemsystem.util.RandomUtil;
import com.yaohun.itemlibrary.api.ItemKuAPI;
import de.tr7zw.itemnbtapi.NBTItem;
import org.bukkit.Bukkit;
import org.bukkit.Material;
import org.bukkit.Sound;
import org.bukkit.entity.Player;
import org.bukkit.event.inventory.InventoryClickEvent;
import org.bukkit.event.inventory.InventoryCloseEvent;
import org.bukkit.inventory.Inventory;
import org.bukkit.inventory.InventoryHolder;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.ItemMeta;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;

public class GemGui implements InventoryHolder, GuiHolder {

    private static int[] SLOT_ARRAY_1 = {0,1,2,3,4,5,6,7,8,9,11,13,15,17,18,19,20,21,22,23,24,25,26};
    private static List<Integer> SLOT_ARRAY_2 = Arrays.asList(10,12,14);
    private static int SLOT_1 = 16;

    private static ItemStack ITEM_1;
    private static ItemStack ITEM_2;
    private static ItemStack ITEM_3;
    private static ItemStack ITEM_4;
    private static ItemStack ITEM_5;
    private static ItemStack ITEM_6;

    private GemInfo cacheGemInfo;
    private List<ItemStack> gemCache = new ArrayList<>();
    private GemCraftData cacheGemCraftData;

    private Inventory inventory;
    private Player player;

    public GemGui(Player player) {
        this.inventory = Bukkit.createInventory(this, 27, "宝石合成");
        this.player = player;
        for (int slot : SLOT_ARRAY_1) {
            inventory.setItem(slot, ITEM_1);
        }
        for (int slot : SLOT_ARRAY_2) {
            inventory.setItem(slot, ITEM_2);
        }
        inventory.setItem(SLOT_1, ITEM_5);
    }

    @Override
    public void open() {
        player.openInventory(inventory);
    }

    @Override
    public void onClick(InventoryClickEvent event) {
        Inventory clickedInventory = event.getClickedInventory();
        int slot = event.getRawSlot();
        event.setCancelled(true);
        if (slot < 27) {
            if (SLOT_ARRAY_2.contains(slot)) {
                int index = getGemSlotIndex(slot);
                if (index != -1 && gemCache.size() > index) {
                    removeGem(index);
                }
            } else if (slot == 16) {
                if (canCraft()) {
                    craft();
                }
            }
        } else {
            ItemStack clickedItem = event.getCurrentItem();
            if (clickedItem == null || clickedItem.getType() == Material.AIR) {
                return;
            }
            GemInfo gemInfo = getGemInfo(clickedItem);
            if (gemInfo == null) {
                player.sendMessage("§8§l[§e§l!§8§l] §7该物品不是宝石");
                return;
            }
            System.out.println(gemInfo);
            if (!GemSystemSetting.hasGemCraftData(gemInfo.getType(), gemInfo.getLevel())) {
                player.sendMessage("§8§l[§e§l!§8§l] §7该物品无法合成");
                return;
            }
            cacheGemCraftData = GemSystemSetting.getGemCraftData(gemInfo.getType(), gemInfo.getLevel());
            if (cacheGemInfo != null && !cacheGemInfo.equals(gemInfo)) {
                player.sendMessage("§8§l[§e§l!§8§l] §7该宝石类型或等级不符");
                return;
            }
            addGem(clickedItem, clickedInventory, event.getSlot());
        }
    }

    @Override
    public void onClose(InventoryCloseEvent event) {
        for (ItemStack itemStack : gemCache) {
            PlayerInventoryUtil.giveItemStack(player, itemStack);
        }
        gemCache.clear();
    }

    @Override
    public Inventory getInventory() {
        return inventory;
    }

    private GemInfo getGemInfo(ItemStack itemStack) {
        NBTItem nbtItem = new NBTItem(itemStack);
        if (nbtItem.hasKey("gemType") && nbtItem.hasKey("gemLevel")) {
            return new GemInfo(nbtItem.getString("gemType"), Integer.parseInt(nbtItem.getString("gemLevel")));
        }
        return null;
    }

    private int getGemSlotIndex(int slot) {
        for (int i = 0; i < SLOT_ARRAY_2.size(); i++) {
            if (slot == SLOT_ARRAY_2.get(i)) {
                return i;
            }
        }
        return -1;
    }

    private void addGem(ItemStack itemStack, Inventory inventory, int slot) {
        if (gemCache.size() >= 3) {
            return;
        }
        int amount = itemStack.getAmount();
        ItemStack gem = itemStack.clone();
        ItemStack itemStack1 = itemStack.clone();
        if (amount > 1) {
            gem.setAmount(1);
            itemStack1.setAmount(amount - 1);
        } else {
            itemStack1 = null;
        }
        if (cacheGemInfo == null) {
            cacheGemInfo = getGemInfo(itemStack);
        }
        gemCache.add(gem);
        update();
        inventory.setItem(slot, itemStack1);
        player.playSound(player.getLocation(), Sound.UI_BUTTON_CLICK, 1, 1);
    }

    private void removeGem(int index) {
        if (!PlayerInventoryUtil.hasFreeSlot(player)) {
            player.sendMessage("§8§l[§c§l!§8§l] §7您的背包空间不足,无法拿下宝石");
            return;
        }
        ItemStack gem = gemCache.get(index);
        PlayerInventoryUtil.giveItemStack(player, gem);
        gemCache.remove(index);
        if (gemCache.size() == 0) {
            cacheGemInfo = null;
            cacheGemCraftData = null;
        }
        update();
    }

    private void update() {
        for (int i = 0; i < SLOT_ARRAY_2.size(); i++) {
            if (gemCache.size() > i) {
                inventory.setItem(SLOT_ARRAY_2.get(i), gemCache.get(i));
            } else {
                inventory.setItem(SLOT_ARRAY_2.get(i), ITEM_2);
            }
        }
        if (gemCache.size() == 3) {
            inventory.setItem(SLOT_1, ITEM_6);
        } else {
            inventory.setItem(SLOT_1, ITEM_5);
        }
    }

    private boolean canCraft() {
        return gemCache.size() == 3;
    }

    private void craft() {
        if (!PlayerInventoryUtil.hasFreeSlot(player)) {
            player.sendMessage("§8§l[§c§l!§8§l] §7您的背包空间不足,无法合成");
            return;
        }
        gemCache.clear();
        boolean success = RandomUtil.random(cacheGemCraftData.getChance());
        ItemStack result = ItemKuAPI.getItems(success ? cacheGemCraftData.getResult() : cacheGemCraftData.getFailResult());
        if (success) {
            player.playSound(player.getLocation(), Sound.ENTITY_PLAYER_LEVELUP, 1, 1);
            player.sendMessage("§8§l[§a§l!§8§l] §7恭喜您,合成成功!");
        } else {
            player.playSound(player.getLocation(), Sound.ENTITY_BLAZE_DEATH, 1, 1);
            player.sendMessage("§8§l[§e§l!§8§l] §7很遗憾,合成失败");
        }
        new GemResultGui(player, result, success).open();
    }

    private class GemInfo {

        private final String type;
        private final int level;

        public GemInfo(String type, int level) {
            this.type = type;
            this.level = level;
        }

        public String getType() {
            return type;
        }

        public int getLevel() {
            return level;
        }

        @Override
        public boolean equals(Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            GemInfo gemInfo = (GemInfo) o;
            return level == gemInfo.level && Objects.equals(type, gemInfo.type);
        }

        @Override
        public int hashCode() {
            int result = Objects.hashCode(type);
            result = 31 * result + level;
            return result;
        }

        @Override
        public String toString() {
            return "GemInfo{" +
                    "type='" + type + '\'' +
                    ", level=" + level +
                    '}';
        }
    }

    static {
        ITEM_1 = new ItemStack(Material.STAINED_GLASS_PANE, 1, (short) 15);
        ITEM_2 = new ItemStack(Material.STAINED_GLASS_PANE, 1, (short) 0);
        ITEM_3 = new ItemStack(Material.STAINED_GLASS_PANE, 1, (short) 5);
        ITEM_4 = new ItemStack(Material.STAINED_GLASS_PANE, 1, (short) 14);
        ITEM_5 = new ItemStack(Material.BARRIER);
        ITEM_6 = new ItemStack(Material.ANVIL);
        ItemMeta itemMeta1 = ITEM_1.getItemMeta();
        itemMeta1.setDisplayName(" ");
        ITEM_1.setItemMeta(itemMeta1);
        ItemMeta itemMeta2 = ITEM_2.getItemMeta();
        itemMeta2.setDisplayName("§a宝石孔");
        ITEM_2.setItemMeta(itemMeta2);
        ItemMeta itemMeta3 = ITEM_3.getItemMeta();
        itemMeta3.setDisplayName(" ");
        ITEM_3.setItemMeta(itemMeta3);
        ItemMeta itemMeta4 = ITEM_4.getItemMeta();
        itemMeta4.setDisplayName(" ");
        ITEM_4.setItemMeta(itemMeta4);
        ItemMeta itemMeta5 = ITEM_5.getItemMeta();
        itemMeta5.setDisplayName("§c请先放入宝石");
        ITEM_5.setItemMeta(itemMeta5);
        ItemMeta itemMeta6 = ITEM_6.getItemMeta();
        itemMeta6.setDisplayName("§f点击合成");
        ITEM_6.setItemMeta(itemMeta6);
    }

}
